#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import pickle
import re
import pandas as pd
from nltk.corpus import stopwords
import time

import hf_extractdiscussions as hf

# specify input values
years = [1891, 1995]
range_years = range(years[0], years[1] + 1)

# paths
#path_start = '/home/lili/NLP_DemocraSci/nlp-democracy/'
path_data = '/data/complete_data/AB/'
path_output = '/data/output/'

# open dictionary of last names from pickle file
with open('/home/lili/nlp-democracy/output/MPs/MPs_lastnames.pickle', 'rb') as f:
    dict_lastnames = pickle.load(f)

# open dictionary of overlaps
with open('/data/complete_data/Results_overlap/DictOverlap1891to1930.pkl', 'rb') as f:
    dict_overlaps_1 = pickle.load(f)
with open('/data/complete_data/Results_overlap/DictOverlap1931to1995.pkl', 'rb') as f:
    dict_overlaps_2 = pickle.load(f)
with open('/data/complete_data/Results_overlap/DictOverlap1991to1995.pkl', 'rb') as f:
    dict_overlaps_3 = pickle.load(f)
dict_overlaps = {**dict_overlaps_1, **dict_overlaps_2, **dict_overlaps_3}
print(dict_overlaps.keys())

# get dictionary of discussions
# -----------------------------

start_time_discussions = time.time()

# list of votation terms
list_votationterms = ['Abstimmung', 'Schlussabstimmung', 'Generalabstimmung', 'Angenommen', 'Abgelehnt', 
                      'Einverstanden', 'Stimmen', '(Eintreten)', '(Nichteintreten)', 
                      'Votation', 'Vote', 'votation', #'(Adopts)', 'adopts', 'adopte', 'rejete', 
                      "D'accord", 'voix']

# list of stopwords
list_stopwords = stopwords.words('german')
list_stopwords.extend(['dass', 'resp', 'daran', 'dr', 'herr', 'herrn', 'hr'])
list_stopwords.extend(stopwords.words('french'))
list_stopwords.extend(['ils', 'les', 'celle'])

print('start to identify discussions of the years', years, '\n\n\n')

# initialize empty dictionary for all documents
dict_docs = {}

# for every year
for year in range_years:
    start_time = time.time()

    str_year = str(year)
    print(year, '\n', 30*'=')
    # initialize empty dictionary for that year
    dict_year = {}
    # extract list of numbers for that year
    list_numbers = next(os.walk(path_data + str_year))[1]#os.listdir(path_data + str_year)
    list_numbers.sort()
    # extract list of lastnames for that year and generate dataframe from it
    lists_lastnames = dict_lastnames[int(year)]
    df_lastnames = hf.get_df_from_lists_names(lists_lastnames)
    # extract overlaps of that year
    dict_overlaps_year = dict_overlaps[year]
    # for each number, i.e. document
    for number in list_numbers:
        path_number = path_data + str_year + '/' + number + '/'
        # if document is a discussion
        if (hf.check_if_discussion(path_number + number + '.xml')) and (number not in ['20032463', '20032952', '20014332']):
            print(number + '\n')
            # get dictionary with text
            dict_text = hf.get_text_onefile(path_number + number + '_datacorr.xml')
            # exclude parts from previous and next document
            if number in dict_overlaps_year:
                dict_text = hf.exclude_overlaps(dict_text, dict_overlaps_year[number])
            # get all discussionstarts
            dict_discussionstarts = hf.get_discussion_starts(dict_text, df_lastnames, list_stopwords, bln_print=True)
            # get votation paragraphs
            dict_votations = hf.get_votations(dict_text, list_votationterms)
            # put all discussions together in dictionary
            dict_discussions, list_keys = hf.get_discussions(dict_text, dict_discussionstarts, dict_votations) 
            # save that discussions dictionary to the yearly dictionary
            dict_year[number] = dict_discussions

            #print('\n\n')
    # save that yearly dictionary to the dictionary for all documents
    dict_docs[year] = dict_year
    # dump that discussions dictionary in the yearly folder
    path_year = path_output + 'AB/' + str_year + '/'
    os.makedirs(path_year, exist_ok=True)
    with open(path_year + 'dict_discussions.pickle', 'wb') as f:
        pickle.dump(dict_year, f)

    print("Time to extract discussions for year %s: %s minutes\n" % (year, (time.time() - start_time)/60))


# dump dictionary of documents to a pickle file
year_start = str(list(dict_docs.keys())[0])
year_end = str(list(dict_docs.keys())[-1])
with open(path_output + 'dict_discussions_' + year_start + '-' + year_end + '.pickle', 'wb') as f:
    pickle.dump(dict_docs, f)

print("Time to extract all discussions: %s minutes\n" % ((time.time() - start_time_discussions)/60))


# Language identification with Luis' method
# -----------------------------------------

print('start to identify languages of the years', years, '\n\n\n')
start_time_languages = time.time()


# initialize empty dictionaries
dict_languages = {}
dict_german = {}
dict_french = {}
dict_italian = {}

# for every year
for year in range_years:
    str_year = str(year)
    start_time = time.time()
    
    # initialize empty dictionaries for that year
    dict_year = {}
    dict_year_german = {}
    dict_year_french = {}    
    dict_year_italian = {}
    print(year)
        
    # load pickle dump for that year
    with open(path_output + 'AB/' + str_year + '/dict_discussions.pickle', 'rb') as f:
        dict_disc_year = pickle.load(f) 
    
    # for every document in that year
    for number in dict_disc_year:   #dict_docs[year]:
        # initiaze empty dictionaries for that document
        dict_number = {}
        dict_number_german = {}
        dict_number_french = {}    
        dict_number_italian = {}
        print(number)
 
        # tokenize discussion
        dict_tokenized = hf.tokenize_dictionary(dict_disc_year[number], hf.tokenizer)
        
        # identify language
        dict_lang = hf.identify_lang(dict_tokenized)
        #print(dict_lang)
        
        # assign language
        for tupel, value in dict_lang.items():
            #print(tupel)
            lang = hf.label_language(value)
            dict_number[tupel] = lang
            if lang == 'german':
                dict_number_german[tupel] = dict_disc_year[number][tupel]
            elif lang == 'french':
                dict_number_french[tupel] = dict_disc_year[number][tupel]
            elif lang == 'italian':
                dict_number_italian[tupel] = dict_disc_year[number][tupel]
            else:
                pass
                #print(lang, value, dict_docs[year][number][tupel])
        
        # add to dictionaries of that year
        dict_year[number] = dict_number
        dict_year_german[number] = dict_number_german
        dict_year_french[number] = dict_number_french
        dict_year_italian[number] = dict_number_italian
        
    # add to overall dictionaries
    dict_languages[year] = dict_year
    dict_german[year] = dict_year_german
    dict_french[year] = dict_year_french
    dict_italian[year] = dict_year_italian

    print("Time to identify languages for discussions of year %s: %s minutes\n" % (year, (time.time() - start_time)/60))
    
    # dump for that year
    with open(path_output + 'AB/' + str_year + '/dict_languages.pickle', 'wb') as f:
        pickle.dump(dict_year, f)
    with open(path_output + 'AB/' + str_year + '/dict_discussions_german.pickle', 'wb') as f:
        pickle.dump(dict_year_german, f)
    with open(path_output + 'AB/' + str_year + '/dict_discussions_french.pickle', 'wb') as f:
        pickle.dump(dict_year_french, f)
    with open(path_output + 'AB/' + str_year + '/dict_discussions_italian.pickle', 'wb') as f:
        pickle.dump(dict_year_italian, f)         

print("Time to identify languages for all discussions: %s minutes\n" % ((time.time() - start_time_languages)/60))
